<?php
/**
 * Description: Prepare tables for Profile: tbl_users, tbl_contact.
 *
 * PHP version 8.0.1
 *
 * @category   webrtc2-profile-tbls.php
 * @package    WP-WebRTC2
 * @author     Oleg Klenitsky <klenitskiy.oleg@mail.ru>
 * @version    1.7.2
 * @license    GPLv2 or later
 */

/**
 * Prepare table for Profile: tbl_users.
 */
function webrtc2_tbl_users() {
  WP_Filesystem();
  global $wp_filesystem;
  ?>
    <div class="caption_tbl">
      <h4>Registered users</h4>
    </div>
    <div class="head_tbl">
      <h4>#</h4>
      <h4>Name</h4>
      <h4>Role</h4>
      <h4>Land</h4>
    </div>
    <div class="body_tbl" id="users_tbody">
    <?php
      $current_user_id = get_current_user_id();
      $hostId_data     = get_userdata( $current_user_id );

      $val_users = get_option( "webrtc2_main_settings" );
      $val_users = isset( $val_users["enabled_for"] ) ? $val_users["enabled_for"] : "registered";
      switch ( $val_users ) {
        case "registered":
          $users = get_users();
          break;
        case "editor":
          $users = get_users("role=editor");
          break;
        case "author":
          $users = get_users("role=author");
          break;
        case "subscriber":
          $users = get_users("role=subscriber");
          break;
        case "contributor":
          $users = get_users("role=contributor");
          break;
      }
      $users_list = get_user_meta($hostId_data->ID, "сontacts_group", true);
      foreach ( $users as $user ) {
        $pos = strpos($users_list, $user->user_login);
        if ($hostId_data->user_login !== $user->user_login && "administrator" !== $user->roles[0] && false === $pos) {
          $avatar       = get_avatar( $user->ID, 20 );
          $user_meta    = get_user_meta( $user->ID );
          $user_country = isset($user_meta["user_country_code"]) ? array_shift( $user_meta["user_country_code"] ) : "";
          $user_city    = isset($user_meta["user_city"]) ? array_shift( $user_meta["user_city"] ) : "";
          ?>
          <div class="tbody_fields">
            <input type="checkbox" name="users_site">
            <div style="display: flex;flex-direction: row;flex:2;">
              <?php echo( $avatar ); ?>
              <label><?php echo __( " " . $user->user_login ); ?></label>
            </div>
            <div style="display: flex;flex-direction: row;flex:2;">
              <label><?php echo __( $user->roles[0] ); ?></label>
            </div>
            <?php
            $img_flag = "";

            if ( is_dir( WP_PLUGIN_DIR . "/wp-webrtc2/" ) ) {
              $dir  = $wp_filesystem->find_folder(WP_PLUGIN_DIR . "/wp-webrtc2/images/flags/");
              $file = trailingslashit($dir) . $user_country . ".gif";

              if ($wp_filesystem->exists($file)) {
                $path_img = set_url_scheme( WP_PLUGIN_URL . "/wp-webrtc2/", "https" ) . "images/flags/" . $user_country . ".gif";
                $img_flag = "<image src='$path_img' >";
              }else{
                $img_flag = "";
                $user_country = "-";
              }
              ?>
              <div>
                <?php echo ( $img_flag ); ?>
                <label title="City:<?php echo __( $user_city ); ?>"><?php echo __( $user_country ); ?></label>
              </div>
            <?php
            }else{
              ?>
              <label>-</label>
              <?php
            }
          ?>
          </div>
          <?php
        }
      }
    ?>
    </div>
    <div class="foot_tbl">
      <input type="text" id="fld_users_search" placeholder="Name...">
      <input type="button" class="btn" id="btn_users_search" onClick="webrtc2_search_user_profile()" value="Search">
    </div>
  <?php
}

/**
 * Prepare table for Profile: tbl_contact.
 */
function webrtc2_tbl_contact() {
  WP_Filesystem();
  global $wp_filesystem;

  $current_user_id = get_current_user_id();
  $hostId_data     = get_userdata( $current_user_id );

  $users_list = get_user_meta($hostId_data->ID, "сontacts_group", true);

  ?>
    <div class="caption_tbl">
      <h4>Contact List</h4>
    </div>
    <div class="head_tbl">
      <h4>#</h4>
      <h4>Name</h4>
      <h4>Role</h4>
      <h4>Land</h4>
    </div>
    <div class="body_tbl" id="contact_tbody">
      <?php
      if ("all_users" !== $users_list) {
        $users_list = explode(";", $users_list);
        foreach ($users_list as $user) {
          if ("" !== $user) {
            $user_data    = get_user_by("login", $user);
            $avatar       = get_avatar( $user_data->ID, 20 );
            $user_meta    = get_user_meta( $user_data->ID );
            $user_country = isset($user_meta["user_country_code"]) ? array_shift( $user_meta["user_country_code"] ) : "";
            $user_city    = isset($user_meta["user_city"]) ? array_shift( $user_meta["user_city"] ) : "";
            ?>
            <div class="tbody_fields">
              <input type="checkbox" name="users_site">
              <div style="display: flex;flex-direction: row;flex:2;">
                <?php echo( $avatar ); ?>
                <label><?php echo __( " " . $user_data->user_login ); ?></label>
              </div>
              <div style="display: flex;flex-direction: row;flex:2;">
                <label><?php echo __( array_shift($user_data->roles )); ?></label>
              </div>
              <?php
              $img_flag = "";

              if ( is_dir( WP_PLUGIN_DIR . "/wp-webrtc2/" ) ) {
                $dir  = $wp_filesystem->find_folder(WP_PLUGIN_DIR . "/wp-webrtc2/images/flags/");
                $file = trailingslashit($dir) . $user_country . ".gif";

                if ($wp_filesystem->exists($file)) {
                  $path_img = set_url_scheme( WP_PLUGIN_URL . "/wp-webrtc2/", "https" ) . "images/flags/" . $user_country . ".gif";
                  $img_flag = "<image src='$path_img' >";
                }else{
                  $img_flag = "";
                  $user_country = "-";
                }
                ?>
                <div>
                  <?php echo ( $img_flag ); ?>
                  <label title="City:<?php echo __( $user_city ); ?>"><?php echo __( $user_country ); ?></label>
                </div>
              <?php
              }else{
                ?>
                <label>-</label>
                <?php
              }
              ?>
            </div>
            <?php
          }
        }
      }
      ?>
    </div>
    <div class="foot_tbl">
      <input type="text" id="fld_contact_search" placeholder="Name...">
      <input type="button" class="btn" id="btn_contact_search" onClick="webrtc2_search_contact_profile()" value="Search">
    </div>
  <?php
}
