/**
 * Description: Creating a WebRTC2AudioMeter.
 *
 * @category    webrtc2-meter-prcsr.js
 * @package     WP-WebRTC2
 * @author      Oleg Klenitsky <klenitskiy.oleg@mail.ru>
 * @version     1.7.2
 * @license     GPLv2 or later
 */

/*jshint esversion: 6 */

"use strict";

/**
 * Description: WebRTC2AudioMeter.
 */
class WebRTC2AudioMeter extends AudioWorkletProcessor {
  constructor() {
    super();

    this.smoothing_factor = 0.8;
    this.masterVolume = 0;
    this.sampleRate = 48000;
    this.updateIntervalInMS = 25;
    this.nextUpdateFrame = this.updateIntervalInMS;
    this.port.postMessage({msg: "WorkletProcessor WebRTC2AudioMeter is work."});
    this.port.onmessage = (event) => {
      // Handling data from the node.
      if (event.data.msg) {
        console.log("WorkletProcessor WebRTC2AudioMeter receive: " + event.data.msg);
      }
    };
  }

  get intervalInFrames() {
    return this.updateIntervalInMS / 1000 * this.sampleRate;
  }

  process(inputs, outputs, parameters) {
    const input = inputs[0];

    if (input.length > 0) {
      const samples = input[0];
      let sum = 0;
      let rms = 0;

      // Calculated the squared-sum.
      for (let i = 0; i < samples.length; ++i) {
        sum += samples[i] * samples[i];
      }
      // Calculate the RMS level and update the masterVolume.
      rms = Math.sqrt(sum / samples.length);
      this.masterVolume = Math.max(rms, this.masterVolume * this.smoothing_factor);

      // Update and sync the masterVolume property with the main thread.
      this.nextUpdateFrame -= samples.length;
      if (this.nextUpdateFrame < 0) {
        this.nextUpdateFrame += this.intervalInFrames;
        this.port.postMessage({vu_audio: this.masterVolume});
      }
    }
    return true;
  }
}
registerProcessor("webrtc2-audio-meter", WebRTC2AudioMeter);
