/**
 * Description: Declaration of global variables.
 *
 * @category    webrtc2-variables.js
 * @package     WP-WebRTC2
 * @author      Oleg Klenitsky <klenitskiy.oleg@mail.ru>
 * @version     1.7.2
 * @license     GPLv2 or later
 */

/*jshint esversion: 6 */

"use strict";

/**
 * Initiator of video-chat or not.
 */
sessionStorage.setItem("webrtc2_initiator", false );

/**
 * Video-chat start or not.
 */
sessionStorage.setItem("webrtc2_chat_start", false );

/**
 * Type of graph stat (graph1 ... graph4).
 */
sessionStorage.setItem("webrtc2_graph_stat", "" );

/**
 * User name of guest of video-chat.
 */
sessionStorage.setItem("webrtc2_guestId", "" );

/**
 * Statistic data for displaying on graph1.
 */
sessionStorage.setItem("webrtc2_bytesReceived_prev_video", 0 );
sessionStorage.setItem("webrtc2_packetsReceived_prev_video", 0 );

/**
 * Statistic data for displaying on graph2.
 */
sessionStorage.setItem("webrtc2_bytesSent_prev_video", 0 );
sessionStorage.setItem("webrtc2_packetsSent_prev_video", 0 );

/**
 * Statistic data for displaying on graph3.
 */
sessionStorage.setItem("webrtc2_bytesSent_prev_transport", 0 );
sessionStorage.setItem("webrtc2_bytesReceived_prev_transport", 0 );

/**
 * Bytes previous of bytesReceived statistics.
 */
sessionStorage.setItem("webrtc2_bytesReceivedPrev", 0 );

/**
 * Time previous of bytesReceived statistics.
 */
sessionStorage.setItem("webrtc2_timestampReceivedPrev", + new Date() );

/**
 * Build last 6 string of fld_chat for videoStreamAll.
 */
sessionStorage.setItem("webrtc2_videoStreamAll_chat", false);

/**
 * Img folder for_fld_file_attach.
 *
 * @var object
 */
var webrtc2_file_select;

/**
 * Object RTCPeerConnection.
 *
 * @var object
 */
var webrtc2_pc;

/**
 * The current number of the drop-down list in the dump window.
 *
 * @var integer
 */
var webrtc2_num_hide = 0;

/**
 * Buffer-array of objects event.candidates
 *
 * @var array
 */
var webrtc2_arr_candidates = [];

/**
 * Data channel for receive or send of messages.
 *
 * @var object
 */
var webrtc2_offerOptions = {
  iceRestart: false,
  offerToReceiveAudio: true,
  offerToReceiveVideo :true,
};

/**
 * Data channel for receive/send of messages.
 *
 * @var object
 */
var webrtc2_dataChannel;

/**
 * Data channel for receive/send stats data.
 *
 * @var object
 */
var webrtc2_statChannel;

/**
 * Data channel for interactive drawing board.
 *
 * @var object
 */
var webrtc2_ctxChannel;

/**
 * MediaRecorder of videoStreamAll = videoStream1 + videoStream2.
 *
 * @var object
 */
var webrtc2_videoStreamAll_MediaRec;

/**
 * Array of videoStreamAll for Blob of webrtc2_videoStreamAll_MediaRec.
 *
 * @var array
 */
var webrtc2_videoStreamAll_BlobsRec = [];

/**
 * Last 6 string of fld_chat for videoStreamAll.
 *
 * @var array
 */
var webrtc2_videoStreamAll_ChatNew = [];

/**
 * Stat WebRTC - inbound_rtp_video.
 *
 * @var object
 */
var webrtc2_inbound_rtp_video = {};

/**
 * Stat WebRTC - inbound_rtp_video of client remote.
 *
 * @var object
 */
var webrtc2_inbound_rtp_video_remote = {};

/**
 * Stat WebRTC - outbound_rtp_video.
 *
 * @var object
 */
var webrtc2_outbound_rtp_video = {};

/**
 * Stat WebRTC - outbound_rtp_video of client remote.
 *
 * @var object
 */
var webrtc2_outbound_rtp_video_remote = {};

/**
 * Stat WebRTC - transport.
 *
 * @var object
 */
var webrtc2_transport = {};

/**
 * Stat WebRTC - transport of client remote.
 *
 * @var object
 */
var webrtc2_transport_remote = {};

/**
 * Stat WebRTC - remote_inbound_rtp.
 *
 * @var object
 */
var webrtc2_remote_inbound_rtp = {};

/**
 * Stat WebRTC - remote_inbound_rtp of client remote.
 *
 * @var object
 */
var webrtc2_remote_inbound_rtp_remote = {};

/**
 * Stat WebRTC - bitrate and fps.
 *
 * @var object
 */
var webrtc2_ticker = {};

/**
 * Stat WebRTC - sound level meters.
 *
 * @var object
 */
var webrtc2_vu_audio = {};

/**
 * Stat WebRTC - bitrate level meters.
 *
 * @var object
 */
var webrtc2_vu_bitrate = {"text" : "unlimited"};

/**
 * Stat WebRTC - change webcam / screen share.
 *
 * @var object
 */
var webrtc2_webcam_screen = {};

/**
 * AudioWorkletNode.
 *
 * @uses In module: webrtc2-service.js
 * @var object
 */
var webrtc2_audio_meter;

/**
 * AudioContext for microphone level.
 *
 * @uses In module: webrtc2-meter-wrklt.js
 * @var object
 */
var webrtc2_audioCtx;

/**
 * BoardCanvas.
 *
 * @uses In module: webrtc2-board.js
 * @var object
 */
var webrtc2_boardCanvas;

/**
 * BoardContext.
 *
 * @uses In module: webrtc2-board.js
 * @var object
 */
var webrtc2_boardCtx;

/**
 * Board remote active.
 *
 * @uses In module: webrtc2-service.js
 * @var object
 */
var webrtc2_board_remote_active = "no";

/**
 * Board local active.
 *
 * @uses In module: webrtc2-service.js
 * @var object
 */
var webrtc2_board_local_active = "no";
